-- by mor2000

-- push object

PushObjectListener = {}

PushObjectListener.RAYCASTDISTANCE = 2
PushObjectListener.IMPULSEPERTON = 2.5
PushObjectListener.POWERUPDURATION = 1000
PushObjectListener.SQUAREBALEMULTIPLIER = 3

PushObjectListener.actionData = InputBinding.actions[InputBinding.PUSHOBJECT_PUSH]

function PushObjectListener:loadMap()
	self.lastIsPressedDtSum = 0
	
	local overlayHeight = 0.12
	self.overlayWidth = overlayHeight / g_screenAspectRatio
	self.handRedOverlay = Overlay:new("handRed", modDirectory.."overlays/hand_red.dds", 0.5-self.overlayWidth/2, 0.5-overlayHeight/2, self.overlayWidth, overlayHeight)
	self.handGreenOverlay = Overlay:new("handGreen", modDirectory.."overlays/hand_green.dds", 0.5-self.overlayWidth/2, 0.5-overlayHeight/2, self.overlayWidth, overlayHeight)
	
	local pixelRedHeight = 0.01
	self.pixelRedOverlay = Overlay:new("pixelRed",  modDirectory.."overlays/pixelRed.dds", 0.5-self.overlayWidth/2, 0.5-overlayHeight/2-pixelRedHeight, self.overlayWidth, pixelRedHeight)
end

function PushObjectListener:deleteMap()
	self.handRedOverlay:delete()
	self.handGreenOverlay:delete()
	self.pixelRedOverlay:delete()
end

function PushObjectListener:update(dt)
	if isPlayerActive() then
		local isDown = PushObjectListener.actionData.lastIsPressed
		if isDown and not self.lastIsPressed then
			self.lastIsPressed = true
		elseif not isDown and self.lastIsPressed then
			self.lastIsPressed = false
			local camx, camy, camz, dirx, diry, dirz, rotY = getCameraPositionAndRotation()
			self:findAndPushObject(camx, camy, camz, dirx, diry, dirz, rotY, self.lastIsPressedDtSum)
			self.lastIsPressedDtSum = 0
		end
		if self.lastIsPressed then
			self.objectToPush = {}
			self.objectToPushFound = false
			local camx, camy, camz, dirx, diry, dirz, rotY = getCameraPositionAndRotation()
			raycastAll(camx, camy, camz, dirx, diry, dirz, "raycastForObjectCallback", PushObjectListener.RAYCASTDISTANCE, self)
			if self.objectToPush.shapeId~=nil then
				self.objectToPushFound = true
			end
			if self.objectToPushFound then
				self.lastIsPressedDtSum = math.max(0, math.min(PushObjectListener.POWERUPDURATION, self.lastIsPressedDtSum + dt))
				self.pixelRedOverlay.width = self.lastIsPressedDtSum/PushObjectListener.POWERUPDURATION * self.overlayWidth
			else
				self.lastIsPressedDtSum = 0
			end
		end
	else
		self.lastIsPressed = false
	end
end

function PushObjectListener:findAndPushObject(camx, camy, camz, dirx, diry, dirz, rotY, dtSum)
	if camx==nil or camy==bil or camz==nil or dirx==nil or diry==nil or dirz==nil or rotY==nil or dtSum==nil then
		return false
	end
	
	if g_server~=nil then
		self.objectToPush = {}
		raycastAll(camx, camy, camz, dirx, diry, dirz, "raycastForObjectCallback", PushObjectListener.RAYCASTDISTANCE, self)
		if self.objectToPush.shapeId~=nil then
			local objectMass = getMass(self.objectToPush.shapeId)
			local impulse = math.sqrt(PushObjectListener.IMPULSEPERTON * objectMass * dtSum/PushObjectListener.POWERUPDURATION * getSquareBaleMultiplier(self.objectToPush.shapeId, PushObjectListener.SQUAREBALEMULTIPLIER))
			local partX, partZ = -math.sin(rotY), -math.cos(rotY)
			local partSum = math.abs(partX) + math.abs(partZ)
			local impulseX = impulse * partX/partSum
			local impulseY = impulse * 0.1
			local impulseZ = impulse * partZ/partSum
			addImpulse(self.objectToPush.shapeId,impulseX,impulseY,impulseZ,self.objectToPush.x,self.objectToPush.y,self.objectToPush.z,false)
		end
	else
		-- do raycast on server
		PushObjectRaycastEvent.sendEvent(camx, camy, camz, dirx, diry, dirz, rotY, dtSum)
	end
end

function PushObjectListener:raycastForObjectCallback(shapeId, x, y, z, distance)
	if getIsObject(shapeId) or getIsLog(shapeId) then
		local t = self.objectToPush
		t.shapeId, t.x, t.y, t.z = shapeId, x, y, z
		return false
	end
	return true
end

function PushObjectListener:draw()
	if self.lastIsPressed then
		if self.objectToPushFound then
			self.handGreenOverlay:render()
			self.pixelRedOverlay:render()
		else
			self.handRedOverlay:render()
		end
	end
end

PushObjectListener.mouseEvent = emptyFunc
PushObjectListener.keyEvent = emptyFunc

addModEventListener(PushObjectListener)
